def calc_3D(in_files,varname,quiet=True,single=False,\
            example_files = None,force_units=True,debug=False,\
            cpt_id = None,reduce_index= None, reduce_method=None,
            reduce_dim = None):
    import budgets_list
    import variables
    import cube_magic as cm
    import iris
    import functions as ff
    import cice_constants as cc
    import copy
    import numpy as np
    import datetime as dt
    import ftimemod

    ref_date = dt.date(1978,9,1)
    ref_units = ref_date.strftime('days since %Y-%m-%d')

    if cpt_id is None:
        cpt_id = budgets_list.cpt_id(varname)
    if example_files is None:
        example_files = [in_files[0]]
    formula = budgets_list.determine_formulae(varname,cpt_id,\
        example_files,debug=debug)
    base_var_list = formula[0]
    expression = formula[1]

    cubedic = {}

    if single:
        np_file_list = np.array(in_files)
        file_dates = np.array([dt.date(ftimemod.fyear(ffile),\
             ftimemod.fmonth(ffile),1)\
             for ffile in in_files])
        unique_fds = np.unique(file_dates)
        looplist = []
        for unique_fd in unique_fds:
            index = np.where(file_dates == unique_fd)
            looplist.append(list(np_file_list[index[0]]))
    else:
        looplist = [in_files]


    calculate_cubelist = iris.cube.CubeList([])
    for filelist in looplist:
        present = []
        for (ivar,base_varname) in enumerate(base_var_list):
            if not quiet:
                print('Extracting variable '+base_varname)

            cubel = cm.find_field(filelist,base_varname)

            if cubel is None:
                print('Could not find component variable '+base_varname)
                present.append(False)
            else:
                if len(cubel) > 1:
                    print('Unable to uniquely identify cube')
                    raise ValueError

                cube = cubel[0]
                if 'forecast_period' in cm.auxcoordnames(cube):
                    cube.remove_coord('forecast_period')

                if force_units:
                    cube.units = '1'

                if reduce_index is not None and len(cube.shape) == 4:
                    cube = cube[:,reduce_index,:,:]

                cubedic[base_varname] = cube
                present.append(True)
                

            expression = expression.replace('var'+str(ivar+1),\
            'cubedic[\''+str(base_varname)+'\']')
            if force_units:
                expression = expression.replace('ccc','cc')
                expression = expression.replace('()','')
        
        if not False in present:
            calculate_cube = eval(expression)
            if reduce_method is not None and reduce_dim is not None:
                calculate_cube = calculate_cube.collapsed(reduce_dim,
                    reduce_method)
            if single:
                if not 'time' in cm.auxcoordnames(calculate_cube) and not \
                       'time' in cm.dimcoordnames(calculate_cube):
                    file_date = dt.date(ftimemod.fyear(filelist[0]),\
                        ftimemod.fmonth(filelist[0]),1)
                    time_point = file_date.toordinal() - ref_date.toordinal()
                    time_coord = iris.coords.AuxCoord(time_point,'time',\
                                  units = ref_units)
                    calculate_cube.add_aux_coord(time_coord)

            calculate_cubelist.append(calculate_cube)

    
    return calculate_cubelist

def ts(cubelist,region_name, gridname,\
       method, area_weights, period, month_mask = None, convert_360_365=False):
    import timemod
    import regions
    import datetime as dt
    import numpy as np
    import numpy.ma as ma
    import copy
    import iris
    import iris.coord_categorisation as icc

    ref_date = dt.date(1978,9,1)

    time_points = []
    data_points = []
    mask_points = []

    for cube in cubelist:
        tc = cube.coord('time')
        datetimelist = [tc.units.num2date(pt) for pt in tc.points]
        if convert_360_365:
            datelist = timemod.d360_to_d365(datetimelist)
        else:
            datelist = [dt.date(ddt.year,ddt.month,ddt.day) for ddt in datetimelist if \
            ddt.year >= period[0] and ddt.year <= period[1]]

        tp_list = [ddt.toordinal() - ref_date.toordinal() for ddt in datelist]

        ccube = copy.deepcopy(cube)
        ccube = timemod.reduce(ccube,startyear=period[0], endyear = period[1])
        if ccube is not None:
            regions.reduce_to_region(ccube,region_name,gridname)

            if month_mask != None:
                if len(ccube.shape) == 3:
                    icc.add_month(ccube,'time')
                    for month in range(1,13):
                        month_name = dt.date(1980,month,1).strftime('%b')
                        i0 = np.where(ccube.coord('month').points == month_name)
                        n0 = i0[0].shape[0]
                        if n0 > 0:
                            iref = np.where(\
                              month_mask.coord('month').points == month_name)
                            amask = np.tile(month_mask[iref[0],:,:].data.mask,(n0,1,1))
                            print(ccube)
                            print(i0)
                            ccube.data.mask[i0[0],:,:] = np.logical_or(amask,\
                              ccube[i0[0],:,:].data.mask)
                else:
                    cube_month = timemod.tmonth(ccube)
                    print(cube_month)
                    month_name = dt.date(1980,cube_month[0],1).strftime('%b')
                    index = np.where(month_mask.coord('month').points == \
                                     month_name)
                    print(index)
                    ccube.data.mask = np.logical_or(month_mask[index[0],:,:],\
                             ccube.data.mask)

                    
            if method == iris.analysis.SUM:
                index = np.where(ccube.data.mask)
                ccube.data.data[index] = 0.

            if len(ccube.shape) == 2:
                aw3 = area_weights
            elif len(ccube.shape) == 3:
                ntime = ccube.shape[0]
                aw3 = np.tile(area_weights,(ntime,1,1))

            ts_cube = ccube.collapsed(['latitude','longitude'],method,\
                weights = aw3)

            if len(ccube.shape) == 3:
                if not ma.is_masked(ts_cube.data):
                    datalist = list(ts_cube.data)
                    masklist = [False] * len(datalist)
                else:
                    datalist = list(ts_cube.data.data)
                    masklist = list(ts_cube.data.mask)
            elif len(ccube.shape) == 2:
                if not ma.is_masked(ts_cube.data):
                    datalist = [ts_cube.data.item(0)]
                    masklist = [False] * len(datalist)
                else:
                    datalist = [ts_cube.data.data.item(0)]
                    masklist = [ts_cube.data.mask.item(0)]


            time_points = time_points + tp_list
            data_points = data_points + datalist
            mask_points = mask_points + masklist

    time_array = np.array(time_points)
    time_sort_index = np.argsort(time_array)
    time_array_sort = time_array[time_sort_index]
    data_array_sort = np.array(data_points)[time_sort_index]
    mask_array_sort = np.array(mask_points)[time_sort_index]

    data_masked_array = ma.masked_array(data_array_sort, mask = mask_array_sort)
    outcube = iris.cube.Cube(data_masked_array)

    ref_units = ref_date.strftime('days since %Y-%m-%d')
    time_coord = iris.coords.DimCoord(time_array_sort,'time',units=ref_units)
    outcube.add_dim_coord(time_coord,0)

    return outcube


def scf(cubelist,period,tolerance = 0.5):
    import timemod
    import numpy as np
    import numpy.ma as ma
    import datetime as dt
    import iris
    
    all_indices = {month: [] for month in range(1,13)}
    for cube in cubelist:
        months = timemod.tmonth(cube)

        for month in range(1,13):
            indices = np.where(months == month)[0]
    
            all_indices[month].append(indices)

        
    counter = np.zeros(12)
    ny, nx = cubelist[0].shape[1:]
    accum_data_array = np.zeros((12,ny,nx))
    accum_mask_array = np.zeros((12,ny,nx))
    for month in range(1,13):
        print('Accumulating seasonal cycle for month '+str(month))
        for (icube,cube) in enumerate(cubelist):
            years = timemod.tyear(cube)
            for item in all_indices[month][icube]:
                if years[item] >= period[0] and years[item] <= period[1]:
                    counter[month-1] += 1
                    if ma.is_masked(cube.data):
                        accum_data_array[month-1,:,:] = \
                          accum_data_array[month-1,:,:] + \
                              cube[item,:,:].data.data
                        accum_mask_array[month-1,:,:] = \
                          accum_mask_array[month-1,:,:] + \
                          cube[item,:,:].data.mask.astype('int')
                    else:
                        accum_data_array[month-1,:,:] = \
                          accum_data_array[month-1,:,:] + cube[item,:,:].data
 
        accum_data_array[month-1,:,:] = \
            accum_data_array[month-1,:,:] / counter[month-1]
        accum_mask_array[month-1,:,:] = \
            accum_mask_array[month-1,:,:] / counter[month-1]

    masked_logical = (accum_mask_array >= tolerance)

    masked_data_array = ma.masked_array(accum_data_array,\
        mask = masked_logical)
    out_cube = iris.cube.Cube(masked_data_array)

    ref_date = dt.date(1978,9,1)
    ref_units = ref_date.strftime('days since %Y-%m-%d')
    time_points = [dt.date(1980,month,1).toordinal() - ref_date.toordinal() \
                    for month in range(1,13)]
    
    time_coord = iris.coords.DimCoord(time_points,'time',units = ref_units)
    out_cube.add_dim_coord(time_coord,0)

    for dc in cubelist[0]._dim_coords_and_dims:
        if 'latitude' in dc[0].standard_name or \
          'longitude' in dc[0].standard_name:
            out_cube.add_dim_coord(*dc)

    for ac in cubelist[0]._aux_coords_and_dims:
        if 'latitude' in ac[0].standard_name or \
          'longitude' in ac[0].standard_name:
            out_cube.add_aux_coord(*ac)

    return out_cube
